#!/usr/bin/env python3

from . import auth
from slack_sdk.errors import SlackApiError
from toolbox.logger import Log

help = "manage channels on Slack"

def setup_args(subparser):
    subparser.add_argument(
        "-a", "--action",
        choices=["create", "addusers", "delusers", "archive", "topic", "unarchive"],
        help='action to take',
        required=True,
        dest='ACTION'
        )
    subparser.add_argument(
        "-n", "--name",
        help='name of the channel to create',
        required=False,
        dest='NAME'
        )
    subparser.add_argument(
        "-c", "--channelid",
        help = 'target channel ID',
        required=False,
        dest='ID'
        )
    subparser.add_argument(
        "-p", "--private",
        action='store_true',
        help='should the new channel be private',
        default=False,
        required=False,
        dest='PRIVATE'
        )
    subparser.add_argument(
        "-u", "--user",
        help='id of the target user',
        required=False,
        dest='USER'
        )
    subparser.add_argument(
        "-t", "--topic",
        help='body of the topic message',
        required=False,
        dest='TOPIC'
        )
    subparser.add_argument(
        "--profile",
        help='name of the user profile to use',
        default='default',
        required=False,
        dest='PROFILE'
        )
    
def main(args):
    SUBCOMMAND = {
        "create": channel_create(args.NAME,args.PRIVATE, args.PROFILE),
        "addusers": channel_user_add(args.ID, args.USER, args.PROFILE),
        "delusers": channel_user_delete(args.ID, args.USER, args.PROFILE),
        "archive": channel_archive(args.ID, args.PROFILE),
        "topic": channel_set_topic(args.ID, args.TOPIC, args.PROFILE),
        "unarchive": channel_unarchive(args.ID, args.PROFILE)
    }
    try: 
        return SUBCOMMAND[args.ACTION]
    except:
        Log.critical(f"Failed to perform action: {args.ACTION}")

def channel_create(channel_name, private, user_profile='default'):
    SLACK_CLIENT = auth.get_slack_session(user_profile)
    try:
        RESPONSE = SLACK_CLIENT.conversations_create(
            name = channel_name,
            is_private = private
        )
        NAME = RESPONSE['channel']['name']
        Log.info(f"created channel {NAME}")
        return RESPONSE
    except SlackApiError:
        Log.critical("Your account/token doesn't have the permission to do that")
    except:
        Log.critical("Failed to create channel")

def channel_user_add(channel_id, user_name, user_profile='default'):
    SLACK_CLIENT = auth.get_slack_session(user_profile)
    try:
        RESPONSE = SLACK_CLIENT.conversations_invite(
            channel = channel_id,
            users = user_name
        )
        NAME = RESPONSE['channel']['name']
        Log.info(f"Invited user to channel {NAME}")
        return RESPONSE
    except SlackApiError:
        Log.critical("Your account/token doesn't have the permission to do that")
    except:
        Log.critical("Failed to add user to channel") 

def channel_user_delete(channel_id, user_name, user_profile='default'):
    SLACK_CLIENT = auth.get_slack_session(user_profile)
    try:
        RESPONSE = SLACK_CLIENT.conversations_kick(
            channel = channel_id,
            user = user_name
        )
        NAME = RESPONSE['channel']['name']
        Log.info(f"Kicked user out of channel {NAME}")
        return RESPONSE
    except:
        Log.critical("Failed to kick user")

def channel_archive(channel_id, user_profile='default'):
    SLACK_CLIENT = auth.get_slack_session(user_profile)
    try:
        RESPONSE = SLACK_CLIENT.conversations_archive(channel = channel_id)
        NAME = RESPONSE['channel']['name']
        Log.info(f"Archived channel {NAME}")
        return RESPONSE
    except SlackApiError:
        Log.critical("Your account/token doesn't have the permission to do that")
    except:
        Log.critical("Failed to archive channel")

def channel_unarchive(channel_id, user_profile='default'):
    SLACK_CLIENT = auth.get_slack_session(user_profile)
    try:
        RESPONSE = SLACK_CLIENT.conversations_unarchive(channel = channel_id)
        NAME = RESPONSE['channel']['name']
        Log.info(f"Un-archived channel {NAME}")
        return RESPONSE
    except SlackApiError:
        Log.critical("Your account/token doesn't have the permission to do that")
    except:
        Log.critical("Failed to unarchive channel")

def channel_set_topic(channel_id, channel_topic, user_profile='default'):
    SLACK_CLIENT = auth.get_slack_session(user_profile)
    try:
        RESPONSE = SLACK_CLIENT.conversations_setTopic(
            channel = channel_id,
            topic = channel_topic
        )
        NAME = RESPONSE['channel']['name']
        Log.info(f"channel topic set to: {channel_topic} in {NAME}")
        return RESPONSE
    except SlackApiError:
        Log.critical("Your account/token doesn't have the permission to do that")
    except:
        Log.critical("Failed to set the channel topic")

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# DO NOT USE. THIS IS VERY, VERY SLOW AND HAS UNKNOWN RESULT. 
# FURTHER TESTING REQUIRED ON NON-PROD SLACK

from . import auth
from toolbox.logger import Log

help = "find an ID for a given channel name (warning, it can be slow!)"

def setup_args(subparser):
    subparser.add_argument(
        "-c", "--channelname",
        help='channel name to use',
        required=True,
        dest='CHANNEL'
        )
    subparser.add_argument(
        "-p", "--profile",
        help='name of the user profile to use',
        required=False,
        default='default',
        dest='PROFILE'
        )
    
def main(args):
    RESPONSE = lookupChannel(args.CHANNEL, args.PROFILE)
    Log.info(f"Found channel name: {RESPONSE['id']}")
    return RESPONSE

def lookupChannel(channel_name, profilename='default'):
    SLACK_CLIENT = auth.get_slack_session(profilename)
    try:
        ALL_CHANNELS = SLACK_CLIENT.conversations_list()
        for CHANNEL in ALL_CHANNELS:
            if CHANNEL['name'] == channel_name:
                return CHANNEL
    except:
        Log.critical("Failed to lookup the channel ID")

if __name__ == "__main__":
    main()


#!/usr/bin/env python3

from . import auth, slack_converter, misc
from slack_sdk import errors
from toolbox.logger import Log

help = "react to a meessage to Slack"

def setup_args(subparser):
    subparser.add_argument(
        "-c", "--channel",
        help='channel or list of channel names (or ids). comma separated',
        required=True,
        dest='CHANNELS'
        )
    subparser.add_argument(
        "-t", "--timestamp",
        help='timestamp of the target message',
        required=True,
        dest='TIMESTAMP'
        )
    subparser.add_argument(
        "-e", "--emoji",
        help='name of the emoji to use',
        required=True,
        dest='EMOJI'
        )
    subparser.add_argument(
        "-p", "--profile",
        help='name of the user profile to use',
        required=False,
        default='default',
        dest='PROFILE'
        )

def main(args):
    RESPONSE = post_slack_reaction(args.CHANNEL, args.TIMESTAMP, args.EMOJI, args.PROFILE)
    Log.info("Reaction posted succesfully")
    return RESPONSE

def post_slack_reaction(channel_id, target_timestamp, emoji, user_profile='default'):
    SLACK_CLIENT = auth.get_slack_session(user_profile)
    try:
        channel_id = slack_converter.convert_slack_names(channel_id)
    except:
        pass

    # convert channel and emoji to strings
    if(type(channel_id)) is tuple:
        channel_id = misc.convertTuple(channel_id)
    elif(type(channel_id)) is list:
        channel_id = ' '.join(channel_id)

    if(type(emoji)) is tuple:
        emoji = misc.convertTuple(emoji)

    #try:
    #    RESPONSE = SLACK_CLIENT.reactions_add(
    #        channel = channel_id,
    #        name = emoji,
    #        timestamp = target_timestamp
    #    )
    #    return RESPONSE
    RESPONSE = SLACK_CLIENT.reactions_add(
         channel = channel_id,
         name = emoji,
         timestamp = target_timestamp
    )
    #    return RESPONSE
    #except errors.SlackApiError:
    #    Log.critical("Unknown emoji name. Reaction not posted.")
    #except Exception:
    #    Log.critical("Failed to post a reaction or reaction already posted")

if __name__ == "__main__":
    main()



import sys, os
try:
    import importlib_resources
except:
    from importlib import resources

def convert_slack_names(target_names):

    TARGET_CHANNELS = []
    MY_RESOURCES = importlib_resources.files("toolbox")
    DATA = (MY_RESOURCES / "slack_channels.lst")

    with open(DATA) as f:
        CONTENTS = f.read()
    for name in target_names:
        for line in CONTENTS.split('\n'):
            if line:
                item = line.split(",")[0]
                value = line.split(",")[1]
                if name == item:
                    TARGET_CHANNELS.append(line.split(",")[1])
    if TARGET_CHANNELS:
        return TARGET_CHANNELS
    else:
        return target_names




import os, getpass
from toolbox.logger import Log
from configstore.configstore import Config
from slack_sdk import WebClient
from toolbox.getpass import getOtherToken

def get_slack_session(user_profile):
    try:
        SLACK_SESSION = WebClient(token=get_slack_token(user_profile))
    except FileExistsError:
        MSG = 'Unable to connect to Slack. Verify your API token'
        LINK = 'https://github.com/stacksc/goat'
        CMD = None
        TITLE = 'GOAT'
        SUBTITLE = 'CRITICAL'
        Log.notify(MSG, TITLE, SUBTITLE, LINK, CMD)
        Log.critical(MSG)
    return SLACK_SESSION

def get_slack_token(user_profile):
    CONFIG = Config('slacktools')
    PROFILE = CONFIG.get_profile(user_profile)

    if PROFILE is None or PROFILE['config'] == {}:
        Log.debug(f"Profile '{user_profile}' not found; creating a new profile")
        while True:
            if user_profile == 'default':
                ENV_TOKEN = os.getenv('SLACK_BOT_TOKEN')
                if ENV_TOKEN is not None:
                    SLACK_TOKEN = ENV_TOKEN
                    break
                Log.debug("Slack token not present in environmental variables")
            Log.info(f"Please enter the API token for Slack profile '{user_profile}'")
            SLACK_TOKEN = getOtherToken("Slack")
            if SLACK_TOKEN != "":
                break
        CONFIGSET = {
            'token': SLACK_TOKEN
        }
        CONFIG.create_profile(user_profile)
        CONFIG.update_config(CONFIGSET, user_profile)
    else:
        try:
            SLACK_TOKEN = PROFILE['config'][user_profile]['token']
        except:
            SLACK_TOKEN = PROFILE['config']['token']
    return SLACK_TOKEN

def get_slackclient_config():
    SLACK_CLIENT = get_slack_session(user_profile='default')
    CONFIG = Config('slacktools')
    RESULT = CONFIG.display_configstore()
    return RESULT



#!/usr/bin/env python3

from . import auth, slack_converter, misc
from toolbox.logger import Log
from slack_sdk import errors

help = "remove a reaction from a meessage in Slack"

def setup_args(subparser):
    subparser.add_argument(
        "-c", "--channelid",
        help='target channel ID',
        required=True,
        dest='CHANNEL'
        )
    subparser.add_argument(
        "-t", "--timestamp",
        help='timestamp of the target message',
        required=True,
        dest='TIMESTAMP'
        )
    subparser.add_argument(
        "-e", "--emoji",
        help='name of the emoji to use',
        required=True,
        dest='EMOJI'
        )
    subparser.add_argument(
        "-p", "--profile",
        help='name of the user profile to use',
        default='default',
        required=False,
        dest='PROFILE'
        )

def main(args):
    RESPONSE = delete_slack_reaction(args.CHANNEL, args.TIMESTAMP, args.EMOJI, args.PROFILE)
    Log.info("Reaction deleted succesfully")
    return RESPONSE

def delete_slack_reaction(channel_id, target_timestamp, emoji, user_profile='default'):
    SLACK_CLIENT = auth.get_slack_session(user_profile)

    try:
        channel_id = slack_converter.convert_slack_names(channel_id)
    except:
        pass

    # convert channel and emoji to strings
    if(type(channel_id)) is tuple:
        channel_id = misc.convertTuple(channel_id)
    elif(type(channel_id)) is list:
        channel_id = ' '.join(channel_id)

    if(type(emoji)) is tuple:
        emoji = misc.convertTuple(emoji)
    try:
        RESPONSE = SLACK_CLIENT.reactions_remove(
            channel = channel_id,
            name = emoji,
            timestamp = target_timestamp
        )
        return RESPONSE
    except errors.SlackApiError as e:
        Log.critical(f"Unknown emoji name. Reaction couldn't be removed: {e}")
    except:
        Log.critical("Failed to remove a reaction or reaction already removed")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

from . import auth, slack_converter
from toolbox.logger import Log

help = "post a meessage to Slack"

def setup_args(subparser):
    subparser.add_argument(
        "-c", "--channel",
        help='channel or list of channel names (or ids). comma separated',
        required=True,
        dest='CHANNELS'
        )
    subparser.add_argument(
        "-m", "--messagetext",
        help='body of the message to send',
        required=True,
        dest='TEXT'
        )
    subparser.add_argument(
        "-r", "--replyto",
        help='timestamp of the message to reply to',
        required=False,
        dest='REPLY'
        )
    subparser.add_argument(
        "-p", "--profile",
        help='name of the user profile to use',
        required=False,
        default='default',
        dest='PROFILE'
        )

def main(args):
    RESPONSES = post_slack_message(args.CHANNELS, args.TEXT, args.REPLY, args.PROFILE)
    for RESPONSE in RESPONSES:    
        Log.info(f"Message sent at {RESPONSE['ts']}")
    return RESPONSES

def post_slack_message(target_channels, message_text, thread_timestamp="", user_profile='default'):
    SLACK_CLIENT = auth.get_slack_session(user_profile)
    TARGET_CHANNELS = []

    try:
        TARGET_CHANNELS = slack_converter.convert_slack_names(target_channels)
    except:
        TARGET_CHANNELS = target_channels

    RESPONSES = []
    if post_prereq(target_channels, thread_timestamp):
        for TARGET_CHANNEL in TARGET_CHANNELS:
            try:
                RESPONSE = SLACK_CLIENT.chat_postMessage(
                    channel = TARGET_CHANNEL,
                    text = message_text,
                    thread_ts = thread_timestamp
                )
                RESPONSES.append(RESPONSE)
            except Exception as e:
                Log.critical(f"Failed to post the message: {e}")
        return RESPONSES
    else:
        Log.critical("Can't reply to a message in multiple channels")

def post_prereq(channels, reply):
# makes sure channels contains just a single channel ID when reply var is set
# as it is impossible to reply to a single message in multiple threads
    if reply == "":
        return True
    else:
        if type(channels) is list:
            return False
        else:
            return True

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import argcomplete, shutil
from .auth import get_slackclient_config
from . import post, react, unpost, unreact, channel
from toolbox.logger import Log
import toolbox.argparse_mod as argparse
from toolbox.argparse_mod import MyFormatter

class Commands():
    def __init__(self):
        self.MAP = {}

    def setup_command(self, GROUP, NAME):
        WIDTH = max(80, shutil.get_terminal_size().columns - 2)
        formatter_class=lambda prog: MyFormatter(prog, max_help_position=40, width=WIDTH)
        MODULE_NAME = NAME.replace('-','_')
        SUBPARSER = GROUP.add_parser(NAME, help=globals()[MODULE_NAME].help, formatter_class=formatter_class)
        globals()[MODULE_NAME].setup_args(SUBPARSER)
        self.MAP.update({NAME: globals()[MODULE_NAME].main})

def init_parsers(COMMANDS):
    LINE='CLI client for Slack'
    PARSER = argparse.ArgumentParser(
        description=LINE, 
        allow_abbrev=True, 
        add_help=True, 
        conflict_handler='resolve'
    )
    PARSER.add_argument(
        "-d","--debug",
        help='debug level; 0 - no output, 1 - normal (default), 2 - verbose', 
        required=False,
        choices=['0','1','2'],
        default='1',
        dest='DEBUG',
    )
    PARSER.add_argument(
        "-c", "--config",
        help="Display the config of slackclient",
        required=False,
        default=False,
        action='store_true',
        dest='SHOW_CONFIG'
    )
    PARSER.add_argument(
        "-p", "--profile",
        help='name of the user profile to use',
        required=False,
        default='default',
        dest='PROFILE'
    )
    SUBPARSERS = PARSER.add_subparsers(dest="CMD", metavar='')
    CMDGROUP_MESSAGES = SUBPARSERS.add_parser_group("\ncommon slack actions include:")
    COMMANDS.setup_command(CMDGROUP_MESSAGES, "post")
    COMMANDS.setup_command(CMDGROUP_MESSAGES, "react")
    COMMANDS.setup_command(CMDGROUP_MESSAGES, "unpost")
    COMMANDS.setup_command(CMDGROUP_MESSAGES, "unreact")
    COMMANDS.setup_command(CMDGROUP_MESSAGES, "channel")
    argcomplete.autocomplete(PARSER)
    return PARSER.parse_args()

def cli():
    COMMANDS = Commands()
    PARSED_ARGS = init_parsers(COMMANDS)
    Log.setup('slacktools', int(PARSED_ARGS.DEBUG))
    if PARSED_ARGS.SHOW_CONFIG:
        get_slackclient_config()
    else:
        try:
            COMMANDS.MAP[PARSED_ARGS.CMD](PARSED_ARGS)
        except KeyError:
            Log.debug(f"No command detected")

if __name__ == "__main__":
    cli()
